/*$

=head1 NAME

Window_key - Window a dataset according to key info

=head1 SYNOPSIS

Window_key  <in.H >out.H  pars

=head1 INPUT PARAMETERS

=over 4

=item verb       -  int 

      [0]  Whether or not to ber berbose

=item maxsize    -  int 

      [20] Maximum size of memory to use for headers

=item synch      -  int 

      [0]  Whether (1) or not (0) to synch data

=item key (1..)  -  char     

      Name of the key the window

=item kindex(1..)-  int      

      Index of key to window

=item maxk(1..)  -  float    

      Max float or int size

=item mink(1..)  -  float    

      Max float or int size

=back

=head1 DESCRIPTION

Windows SEP3D headers dataset according to key values

=head1 COMMENTS

Relies on superset
Use to be written in ratfor77

=head1 SEE ALSO

L<Window3d>

=head1 CATEGORY

B<util/headers>

=cut


>*/ 
/*
-------------------------------------------------

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Sat Jul 31 18:18:18 PDT 1999

Purpose: 

*/	 

#include <superset.h>
#if defined(HAVE_LIMITS_H)
#include <limits.h>
#endif

#ifndef INT_MAX
#define INT_MAX  2147483647
#endif
#ifndef INT_MIN
#define INT_MIN  -2147483647+1
#endif

#define STRING_SIZE 256
#define IS_INT 1
#define IS_FLOAT 0
int verb;

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int get_pars(int *synch, int *nkeys,int *maxsize, int *n, int *n1, float *o1, float *d1, char label[STRING_SIZE], char unit[STRING_SIZE], int *esize);
int get_min_max(int nkeys,int *nlmt, int *key_num, int *key_type,int *int_min,
 int *int_max,float *float_min, float *float_max);
int helix_out(int n1,float o1,float d1,char label[STRING_SIZE],
char  unit[STRING_SIZE],int n,int synch);
_XFUNCPROTOEND
#else 
int get_pars();
int get_min_max();
int helix_out();
#endif

#define MY_SEP_DOC \
 sep_add_doc_line("NAME");\
 sep_add_doc_line("    Window_key - Window a dataset according to key info");\
 sep_add_doc_line("");\
 sep_add_doc_line("SYNOPSIS");\
 sep_add_doc_line("    Window_key <in.H >out.H pars");\
 sep_add_doc_line("");\
 sep_add_doc_line("INPUT PARAMETERS");\
 sep_add_doc_line("    verb - int");\
 sep_add_doc_line("              [0]  Whether or not to ber berbose");\
 sep_add_doc_line("");\
 sep_add_doc_line("    maxsize - int");\
 sep_add_doc_line("              [20] Maximum size of memory to use for headers");\
 sep_add_doc_line("");\
 sep_add_doc_line("    synch - int");\
 sep_add_doc_line("              [0]  Whether (1) or not (0) to synch data");\
 sep_add_doc_line("");\
 sep_add_doc_line("    key (1..) - char");\
 sep_add_doc_line("              Name of the key the window");\
 sep_add_doc_line("");\
 sep_add_doc_line("    kindex(1..)- int");\
 sep_add_doc_line("              Index of key to window");\
 sep_add_doc_line("");\
 sep_add_doc_line("    maxk(1..) - float");\
 sep_add_doc_line("              Max float or int size");\
 sep_add_doc_line("");\
 sep_add_doc_line("    mink(1..) - float");\
 sep_add_doc_line("              Max float or int size");\
 sep_add_doc_line("");\
 sep_add_doc_line("DESCRIPTION");\
 sep_add_doc_line("    Windows SEP3D headers dataset according to key values");\
 sep_add_doc_line("");\
 sep_add_doc_line("COMMENTS");\
 sep_add_doc_line("    Relies on superset Use to be written in ratfor77");\
 sep_add_doc_line("");\
 sep_add_doc_line("SEE ALSO");\
 sep_add_doc_line("    Window3d");\
 sep_add_doc_line("");\
 sep_add_doc_line("CATEGORY");\
 sep_add_doc_line("    util/headers");\
 sep_add_doc_line("");
#include <sep.main> 
MAIN()
{ 
int maxsize,synch,edit,n_keys;
int nkeys,ndim,nlimits,i;
int *int_header,*int_min,*int_max,*key_num,*key_type,*drn_in,*drn_out;
float *float_min,*float_max;
float *float_header,o1,d1,*float_buf;
int *int_buf,*buf_out,n,esize,n1,nout,nblock;
int nwind[2],fwind[2],jwind[2];
char *buffer;
char keyname[1024],keytype[1024],keyfmt[1024],label[STRING_SIZE],unit[STRING_SIZE];
char temp_ch[1024],temp2_ch[1024];
int nlimit,max_block,read,one,nh,i2,logic,i1,write_data;

getch_add_string("gff_in=\"-1\"");
init_3d();

if(SUCCESS!=get_pars(&synch,&nkeys,&maxsize,&n,&n1,&o1,&d1,label,unit,&esize))
	seperr("trouble grabbing pars \n");

sep_get_number_keys("in",&n_keys);
if(0==sep_get_key_index("in","data_record_number",&i))
  n_keys-=1;



int_min=(int*)malloc(sizeof(int)*nkeys);
int_max=(int*)malloc(sizeof(int)*nkeys);
key_num=(int*)malloc(sizeof(int)*nkeys);
key_type=(int*)malloc(sizeof(int)*nkeys);
float_min=(float*)malloc(sizeof(int)*nkeys);
float_max=(float*)malloc(sizeof(int)*nkeys);

if(SUCCESS!=get_min_max(nkeys,&nlimit,key_num,key_type,int_min,int_max,
  float_min, float_max)) seperr("trouble grabbing min max \n");

if(SUCCESS!=helix_out(n1,o1,d1,label,unit,n,synch))
	seperr("trouble grabbing pars \n");


maxsize=maxsize*1000000;
if(synch==0){
	max_block=maxsize/2/((n_keys+3)*4);
}
else{
	 max_block=maxsize/2/((n_keys+3)*4 + n1 *esize);
}
if(max_block > n) max_block=n;
if(n< 0) seperr("Can't even hold a single trace in memory\n");
if(synch==1) buffer=(char*) malloc(sizeof(char)*esize*n1*max_block);



buf_out=(int*) malloc(sizeof(int)*n_keys*max_block);
drn_in=(int*) malloc(sizeof(int)*max_block);
drn_out=(int*) malloc(sizeof(int)*max_block);

read=0;one=1;nout=0;
nwind[0]=n1; fwind[0]=0; jwind[0]=1; jwind[1]=1;
while(read < n){
	edit=0;
	nblock=MIN(n-read,max_block);
	if(SUCCESS!=sep3d_read_headers("in","in",&nblock,&read,&one,&nh))
		seperr("trouble grabbing headers \n");
	if(SUCCESS!=sep3d_grab_drn("in",drn_in))
		seperr("trouble grabbing drn \n");
	int_buf= sep3d_grab_header_pointer("in");
	float_buf=(float*)sep3d_grab_header_pointer("in");

	for(i2=0; i2 < nh;i2++){
		logic=0; i1=0; while(logic==0 && i1 < nlimit){
			if(key_type[i1] == IS_INT){
				if(int_buf[i2*n_keys+key_num[i1]-1] < int_min[i1] ||
				int_buf[i2*n_keys+key_num[i1]-1] > int_max[i1]) logic=1;
			}
			else{
				if(float_buf[i2*n_keys+key_num[i1]-1] < float_min[i1] ||
				float_buf[i2*n_keys+key_num[i1]-1] > float_max[i1]) logic=1;
			}
			i1+=1;
		}
		if(logic==0) { 
			memcpy((void*) (&buf_out[n_keys*edit]),(const void*)(&float_buf[i2*n_keys]),
				n_keys*4);
			drn_out[edit]=drn_in[i2];
			edit++;
		}
	}
	if(verb)fprintf(stderr,"Working from %d to %d of %d found %d valid headers\n",
		read,read+nblock,n,edit);
	if(read+nblock==n){
		if(synch==1) sep3d_rite_file_stat("out",1,1);
		else sep3d_rite_file_stat("out",0,1);
		if(SUCCESS!=sep3d_set_ntraces("out",edit+nout))
			seperr("trouble setting ntraces \n");
		if(SUCCESS!=sep3d_rite_ntraces("out","out"))
			seperr("trouble writing ntraces \n");
		if(synch==0) putch("n2","d",&n);
		sep_3d_close();
	}
	if(edit>0){
		if(SUCCESS!=sep3d_set_header_pointer("out",edit,buf_out))
		seperr("trouble setting header pointer \n");
		if(synch==1){
			if(SUCCESS!=sep3d_set_inorder("out"))
				seperr("trouble setting in order \n");
			if(SUCCESS!=sep3d_read_list("in",n1,n1,0,1,esize,edit,drn_out,buffer))
				seperr("trouble reading in headers \n");
			write_data=1;
		}
		else{
			if(SUCCESS!=sep3d_set_nh("out",edit))
				seperr("trouble setting drn \n");
			if(SUCCESS!=sep3d_set_drn("out",drn_out))
				seperr("trouble setting drn \n");
			write_data=0;
		}
		nwind[1]=edit; fwind[1]=nout;
		if(SUCCESS!=sep3d_rite("out","out",nwind,fwind,jwind,buffer,edit,write_data,
     1,0)) seperr("trouble writting out data \n");
	
		if(0!=sep3d_nullify_header_pointer("out"))
		seperr("trouble nullifying header pointer \n");
	}
	nout+=edit; read+=nblock;
}
   
	
return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int get_pars(int *synch, int *nkeys, int *maxsize, int *n, int *n1, float *o1, float *d1, char label[STRING_SIZE], char unit[STRING_SIZE], int *esize)
_XFUNCPROTOEND
#else
int get_pars(synch,nkeys,maxsize,n,n1,o1,d1,label,unit,esize)
int *synch, *nkeys, *n, *n1, *esize,*maxsize; 
float *o1,  *d1;
char label[STRING_SIZE],unit[STRING_SIZE];
#endif
{
int i,tempi,i2;
int ndim;
char temp_ch[STRING_SIZE],temp2_ch[STRING_SIZE];
float o,d;

if(0==getch("verb","d",&verb)) verb=0;
if(0==getch("maxsize","d",maxsize)) *maxsize=20;
if(0==getch("synch","d",synch)) *synch=0;
 
if(SUCCESS!=sep3d_tag_init("in","in","INPUT"))
  seperr("trouble initializing tag in \n");

if(SUCCESS!=sep3d_grab_nkeys("in",nkeys))
	seperr("trouble grabbin number of keys \n");
 
if(SUCCESS!=sep3d_grab_axis("in",1,n1,o1,d1,label,unit))
	seperr("trouble grabbing axis 1\n");

if(SUCCESS!=sep3d_grab_ndims("in",&ndim))
	seperr("trouble grabbing number of dimnensions \n");
 
*n=1;
for(i2=2; i2 <= ndim;i2++){
	if(SUCCESS!=sep3d_grab_axis("in",i2,&tempi,&o,&d,temp_ch,temp2_ch))
		seperr("trouble grabbing axis %d \n",i2);
	*n=(*n)*tempi;
}
if(0==hetch("esize","d",esize)) *esize=4;

return(SUCCESS);
} 

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int get_min_max(int nkeys,int *nlmt, int *key_num, int *key_type,int *int_min,
 int *int_max,float *float_min, float *float_max)
_XFUNCPROTOEND
#else
int get_min_max(nkeys,nlmt,key_num,key_type,int_min,int_max,float_min,float_max)
int *nlmt,*key_num, *key_type,*int_min,*int_max,nkeys;
float *float_min,  *float_max;
#endif
{
char temp_ch[STRING_SIZE],keyname[STRING_SIZE],big[1024];
char type[STRING_SIZE],fmt[STRING_SIZE];
int i,logic,drn;

if(0!=sep_get_key_index("in","data_record_number",&drn)) drn=nkeys+1;
	

i=1;logic=0;*nlmt=0;
while(logic==0 && i<=nkeys){
	sprintf(temp_ch,"key%d",i);
	if(0==getch(temp_ch,"s",keyname)){ /*Couldn't find keyname */
		sprintf(temp_ch,"kindex%d",i);
		if(0==getch(temp_ch,"d",&key_num[i-1])) logic=1;
		else{
			if(key_num[i-1] > drn) key_num[i-1]-=1; /*correct for drn not in headers*/
			if(key_num[i-1]<1 || key_num[i-1] > nkeys)
				seperr("invalid key index \n");
			*nlmt+=1;
		}
	}
	else{
		if(0!=sep3d_grab_key_index("in",keyname,&key_num[i-1]))
			seperr("could not find key %s \n");
		*nlmt+=1;
	}
	if(logic==0){ /* we found a key */
		if(0!=sep3d_grab_key("in",key_num[i-1],keyname,type,fmt))
			seperr("trouble grabbing key \n");
		if(0==strcmp(type,"scalar_float")){
			 key_type[i-1]=IS_FLOAT;
			 sprintf(temp_ch,"mink%d",i);
			 if(0==getch(temp_ch,"f",&float_min[i-1])) float_min[i-1]=-9e20;
			 sprintf(temp_ch,"maxk%d",i);
			 if(0==getch(temp_ch,"f",&float_max[i-1])) float_max[i-1]=9e20;
				sprintf(big,"Key %s :keynum=%d min=%f max=%f \n",
        keyname,key_num[i-1],float_min[i-1],float_max[i-1]);
			 if(verb==1) fprintf(stderr,"%s\n",big);
			 putlin(big);
		}
		else{
		 	key_type[i-1]=IS_INT;
			 sprintf(temp_ch,"mink%d",i);
			 if(0==getch(temp_ch,"d",&int_min[i-1])) int_min[i-1]=INT_MIN;
			 sprintf(temp_ch,"maxk%d",i);
			 if(0==getch(temp_ch,"d",&int_max[i-1])) int_max[i-1]=INT_MAX;
			 sprintf(big,"Key %s :keynum=%d min=%d max=%d \n",
        keyname,key_num[i-1],int_min[i-1],int_max[i-1]);
			 if(verb) fprintf(stderr,"%s\n",big);
			 putlin(big);
		}
	}
	i+=1;
}

return(SUCCESS);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int helix_out(int n1,float o1,float d1,char label[STRING_SIZE],
char  unit[STRING_SIZE],int n,int synch)
_XFUNCPROTOEND
#else
int helix_out(n1,o1,d1,label,unit,n,synch)
int n1;float o1;float d1;char label[STRING_SIZE];
char  unit[STRING_SIZE];int n;int synch;
#endif
{
char temp_ch[STRING_SIZE];

if(SUCCESS!=sep3d_set_ndims("in",2))
	seperr("trouble setting ndims \n");
if(SUCCESS!=sep3d_set_axis("in",1,n1,o1,d1,label,unit))
	seperr("trouble setting axis 1\n");
if(SUCCESS!=sep3d_set_axis("in",2,n,1.,1.,"trace number","none"))
	seperr("trouble setting axis 2\n");

if(SUCCESS!=sep3d_grab_file_type("in",temp_ch))
	seperr("trouble grabbing file type \n");

if(0!=strcmp(temp_ch,"HEADER"))
	seperr("Input must be of type headers , %s \n",temp_ch);


if(SUCCESS!=sep3d_struct_init("in","out","OUTPUT"))
	seperr("trouble creating output");

if(SUCCESS!=sep3d_set_nh("out",1))
	seperr("trouble setting number headers \n");
if(synch==1){
	if(SUCCESS!= sep3d_set_inorder("out"))
		seperr("trouble setting in order \n");
}
if(SUCCESS!=sep3d_rite_format("out","out"))
	seperr("trouble writing out format \n");

if(SUCCESS!=sep3d_clear_headers("out"))
	seperr("trouble clearing headers \n");

if(synch==0) sep_copy_data_pointer("in","out");

return(SUCCESS);
}

