!$
!$=head1 NAME
!$
!$veltran - velocity transform with anti-aliasing and sqrt(  i omega)
!$
!$
!$=head1 SYNOPSIS
!$
!$C<call veltran(adj,add,psun,s02,anti,t0,dt,x0,dx,s0,ds,nt,nx,ns,model,data)>
!$
!$=head1 INPUT PARAMETERS
!$
!$=over 4
!$
!$=item adj   - int
!$
!$      0- (slow->offset), 1- (offset->slow)
!$
!$=item add   - int
!$
!$      0- erase output, 1 save output
!$
!$=item psun  - int
!$
!$      0-modeling, 1-pseudounitary, 2-vel tran
!$
!$=item s02   - real
!$
!$      ????
!$
!$=item anti  - real
!$
!$      Triangle size
!$
!$=item t0    - real
!$
!$      Initial time
!$
!$=item dt    - real
!$
!$      Sampling in time
!$
!$=item x0    - real
!$
!$      First offset
!$
!$=item dx    - real
!$
!$      Offset sampling
!$
!$=item s0    - real
!$
!$      First slowness
!$
!$=item ds    - real
!$
!$      Sampling in slowness
!$
!$=item nt    - int
!$
!$      Number of time samples
!$
!$=item nx    - int
!$
!$      Number of offset
!$
!$=item ns    - int
!$
!$      Number of slownesses
!$
!$=item modl  - real(nt,ns)
!$
!$      time,slowness panel
!$
!$=item data  - real(nt,nx)
!$
!$      time,offset panel
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Velocity transform with anti-aliasing
!$
!$
!$=head1  COMMENTS
!$
!$Requires trimo, null, adjnull
!$
!$=head1 LIBRARY
!$
!$B<sepvelanf>
!$
!$=cut
!$
subroutine veltran(adj,add,psun,s02,anti,t0,dt,x0,dx,s0,ds,nt,nx,ns,model&
  &,data)
integer it,ix,is,  adj,add,psun,                           nt,nx,ns
real x, s, wt,               s02,anti,t0,dt,x0,dx,s0,ds,model(nt,ns)&
  &,data(nt,nx)
real slow(nt), half(nt,nx)
call null(               half,nt*nx)
call adjnull(      adj,add,                             model,nt*ns,&
  & data,nt*nx)
if ( adj .ne. 0) then
  do ix = 1, nx
    call halfdifa( adj,   0, nt, half(1, ix), data(1, ix) )
  end do
end if
do is= 1, ns  
  s = s0 + (is-1) * ds
  do it=1,nt  
    slow(it) = s
  end do 
  do ix= 1, nx  
    x = x0 + (ix-1) * dx
    if ( psun .eq. 2 ) then
      wt =       abs( s * x)
! vel tran
    else if ( psun .eq. 1 ) then
      wt = sqrt( abs( s * x))
! pseudounitary
    else
      wt = 1.
    end if 
! modeling
    call trimo( adj, 1, t0,dt,dx, x, nt,slow, s02,                    &
      &     wt , anti,  model(1,is),  half(1,ix))
  end do
end do 
if ( adj .eq. 0) then
  do ix = 1, nx
    call halfdifa( adj, add, nt, half(1, ix), data(1, ix) )
  end do
end if
return
end  
