!$
!$
!$=head1 NAME
!$
!$Transf - Transposes and FFTs a dataset
!$
!#$=head1 SYNOPSIS
!$
!$Transf  <in.H >out.H
!$
!$=head1 INPUT PARAMETERS
!$
!$=over 4
!$
!#$=item f_min - float
!$
!$	[0]: Minimum frequency (only needed for forward op)
!$
!$=item f_min1 - float
!$
!$	[f_min]: Minimum frequency with 100% pass (only needed for forward op)
!$
!$=item f_max - float
!$
!$	[.5/d1]: Maximum frequency  (only needed for forward op)
!$
!$=item f_max1 - float
!$
!$	[f_max]: Maximum frequency with 100% pass (only needed for forward op)
!$
!$=item shift2 - integer
!$
!$	[0]: How many n=1 axes to introduce or remove after the frequency axis
!$
!$=item maxsize - integer
!$
!$	[50]: Maximum megabytes to use
!$
!$=item verb  - logical
!$
!$	[n]: Whether or not to be verbose
!$
!$=item  wei  - logical
!$
!$	[false]: Do WEI style transforms
!$
!$=back
!$
!$
!$=head1 DESCRIPTION
!$
!$Transforms a dataset (and its inverse) from (nt,nx,...) to (nx ...,nt)
!$
!$=head1 NOTES
!$
!$ Forward/Inverse determined from esize (4-forward, 8-inverse)
!$
!$ WARNING WARNING WARNING: Unpipable
!$
!$=cut
!>
!-#------------------------------------------------
!
!Author: Robert Clapp, ESMB 463, 7230253
!
!Date Created:Fri Sep 22 12:42:13 PDT 2000
!
!Purpose:
!
program Transf 
  use sep
  use cos_taper_mod
  implicit none
  integer :: i,ib,nt
  integer :: nb !block size
  integer :: shift2 !whether or not shift the second axis
  integer :: maxsize !maximummemory
  integer :: ir,ii   !imag and real pointers
  integer :: i_f_min,i_f_max
  integer :: nh !helix coordinate system
  integer :: nf !number of frequencies
  integer :: done,nbs
  logical :: verb,inv
  real    :: df !sampling of frequencies
  real    :: f_min,f_max
  real    :: o,d,dt,of,ot,w,f_min1,f_max1
  integer :: n,ndim,esize,np,ierr
  integer :: ng(2),fw(2),jw(2),nw(2),j
  character(len=128) :: label
  real, allocatable :: block(:,:),temp(:)
  complex,allocatable :: block_out(:),block_c(:,:),block_t(:),scalet(:&
    &),tt(:)
  integer,external :: sep_get_number_data_axes,srite_window&
    &,sreed_window
  integer :: sign
  logical :: wei
  real :: pi
  real arg
  complex, allocatable :: shift(:)
!READ IN PARAMS
  integer fetch,getch,hetch,putch
  call initpar()
  call doc('SOURCE')
    if (0>= getch('maxsize','d',maxsize )) then
      maxsize = 50
    end if
    if (0.ne.putch('Read  from param: #maxsize ','d',maxsize)) then
      call erexit('Trouble writing maxsize to history')
    end if
    if (0>= getch('shift2','d',shift2 )) then
      shift2 = 0
    end if
    if (0.ne.putch('Read  from param: #shift2 ','d',shift2)) then
      call erexit('Trouble writing shift2 to history')
    end if
  call from_param("verb",verb,.false.)
    if (0>= hetch('esize','d',esize )) then
      esize = 4
    end if
  call from_param("wei",wei,.false.)
  if (esize.eq.4) then
    inv=.false.
  else if (esize.eq.8) then
    inv=.true.
  else
    call seperr("esize must be 4 or 8")
  end if 
  ierr= sep_get_number_data_axes("in",ndim)
  if (wei .and. shift2.eq.0) then
    ndim=5
  else if (wei) then
    ndim=4
  end if
  if (verb .and. wei) then
    write(0,*) "Using  wei conventions"
  end if
  nh=1
  do i=1,ndim 
    call sep_get_data_axis_par("in",i,n,o,d,label)
    if (inv) then
      if (i.eq.ndim) then
        df=d
        of=o
        nf=n
          if (0.ne. putch('esize','d',4 )) then
            call erexit('Trouble writing  esize to history  ')
          end if
        nt=1
        do while (nf>nt)
          nt=nt*2
        end do 
        nt=nt*2
        ot=0.
        dt=1./(nt*df)
        call sep_put_data_axis_par("out",1,nt,ot,dt,"Time")
      else if (shift2>0 .and. shift2+1 <= i .and. n.ne.1) then
        call seperr("shift2 specified but the axis length not 1")
      else
        call sep_put_data_axis_par("out",i+1-shift2,n,o,d,"Label")
        nh=nh*n
      end if
    else
      if (i.eq.1) then
          if (0>= fetch('f_min','f',f_min )) then
            f_min = 0.
          end if
          if (0>= fetch('f_max','f',f_max )) then
            f_max = .5/d
          end if
        ot=o
        dt=d
        nt=n
        nf=.5/dt
        df=1./(dt*nt)
        call from_param("f_min1",f_min1,f_min+df*2)
        call from_param("f_max1",f_max1,f_max-df*3)
        of=0.
        i_f_min=max(1,nint(f_min/df))
        i_f_max=min(int(nt/2),nint(f_max/df))
        nf=i_f_max-i_f_min+1
        f_min = (i_f_min-1)*df
        f_max = (i_f_max-1)*df
          if (0.ne. putch('esize','d',8 )) then
            call erexit('Trouble writing  esize to history  ')
          end if
        call sep_put_data_axis_par("out",ndim+shift2,nf,f_min,df,"Frequency"&
          &)
        if (shift2>0) then
          do j=1,shift2 
            call sep_put_data_axis_par("out",1+j,1,0.,1.,"none")
          end do
        end if
      else
        if (i.eq.2 .and. shift2 >0) then
          call sep_put_data_axis_par("out",1,n,o,d,label)
        else
          call sep_put_data_axis_par("out",i-1+shift2,n,o,d,label)
        end if 
        nh=nh*n
      end if
    end if
  end do 
  if (.not. inv) then
    call init_cos_taper(nf,f_min,df,f_min,f_min1,f_max,f_max1)
  end if
  call hclose()
!COMPUTE PARAMS
  nb=min(nh,maxsize*1000000/nt/8)
  jw=1
  nw=1
  fw=0
  if (.not. inv) then
    ng=(/nh,nf/)
  else
    ng=(/nh,nf/)
  end if 
  allocate(scalet(nt))
  if (.not. inv) then
    w=8.*atan(1.) *ot
    do i=1,nt  
      scalet(i)=cexp(cmplx(0.,w*((i-1)*df+of)))
    end do
  else
    w=8.*atan(1.) *of
    do i=1,nt  
      scalet(i)=cexp(cmplx(0.,w*((i-1)*dt+ot)))
    end do
  end if 
!ALLOCATE MEMORY
  if (.not. inv) then
    allocate(block(nt,nb),temp(nt*2+16),block_out(nb))
    allocate(tt(nf))
  else
    allocate(block_c(nb,nf),block_t(nt))	 !scalet(nt)
  end if 
!INITIALZIE
  if (.not. inv) then
    call RFFTI(nt,temp)
  end if
!MAIN LOOP
  done=0
  do while (done < nh)
    nbs=min(nh-done,nb)
    if (.not. inv) then
      call sreed("in",block,nbs*4*nt)
    else
      fw(1)=done
      nw(1)=nbs
      nw(2)=nf
      if (0.ne. sreed_window("in",2,ng,nw,fw,jw,8,block_c( 1:nbs, :)))&
        & then
        write(0,*) "ng",ng 
        write(0,*) "nw",nw 
        write(0,*) "fw",fw 
        write(0,*) "jw",jw 
        call seperr("trouble reading window (1)")
      end if
    end if 
    if (verb) then
      write(0,*) "Finished reading ",done," to ",done+nbs
    end if
    if (.not. inv) then
!   write(0,*) "Check block",sum(block)
      do i=1,nbs
        call RFFTF(nt,block(1,i),temp)
      end do 
      if (verb) then
        write(0,*) "Finished transform ",done," to ",done+nbs
      end if
      fw(1)=done
      nw(1)=nbs
      do i=1,nf 
        j=i+i_f_min-1
        fw(2)=i-1
        if (j.eq.1) then
          ir=1
          ii=nt
        else
          ir=(j-1)*2
          ii=ir+1
        end if 
        if (wei) then
          do ib=1,nbs 
            block_out(ib)=scalet(j)*cmplx(block(ir,ib),-block(ii,ib))&
              &*cos_taper_filter(i)
          end do
        else
          do ib=1,nbs 
            block_out(ib)=scalet(j)*cmplx(block(ir,ib),block(ii,ib))&
              &*cos_taper_filter(i)
          end do
        end if 
!   write(0,*) "Check rite",sum(block_out)
        if (0.ne. srite_window("out",2,ng,nw,fw,jw,8,block_out)) then
          write(0,*) "trouble writing out ng",ng
          write(0,*) "trouble writing out nw",nw
          write(0,*) "trouble writing out fw",fw
          write(0,*) "trouble writing out jw",jw
          call erexit("")
        end if
      end do
    else
      do i=1,nbs 
        if (wei) then
          block_t(1:nf)=cmplx(real(block_c(i,:)),-aimag(block_c(i,:)))
        else
          block_t(1:nf)=block_c(i,:)
        end if 
        block_t(nf+1:nt)=cmplx(0.,0.)	
        call ftu(1.,nt,block_t)
        block_t=block_t  *scalet
        call srite("out",real(block_t),nt*4)
      end do
    end if 
    if (verb) then
      write(0,*) "Finished write ",done," to ",done+nbs
    end if
    done=		done+nbs
  end do 
end program  
